if [ -z "$1" ]
  then
  QUARTUS_NAME=ghrd_10as066n2
else
  QUARTUS_NAME=$1
fi

if [ -z "$2" ]
  then 
  QSYS_NAME=qsys_top
else
  QSYS_NAME=$2
fi

fail () {
msg=$*
echo "Error: $msg"
exit -1
}

echo "Quartus name: $QUARTUS_NAME";
echo "Qsys name   : $QSYS_NAME";

# generate altsource_probe for hps_reset
#qsys-generate --synthesis=VERILOG ip/altsource_probe/hps_resets.qsys || fail "HPS RESETS Qsys system generation failed"

tclsh create_ghrd_top.tcl qsys_name $QSYS_NAME fpga_dp 1 pr_enable 1|| fail "System top creation failed"
quartus_sh --script=create_ghrd_quartus.tcl projectname $QUARTUS_NAME qsysname $QSYS_NAME qsys_pro 1 fpga_dp 1 pr_enable 1|| fail "$QUARTUS_NAME Quartus project creation failed"
qsys-script --script=create_ghrd_qsys.tcl --quartus-project=$QUARTUS_NAME --cmd="set qsys_name $QSYS_NAME; set qsys_pro 1; set fpga_dp 1; set pr_enable 1" || fail "$QSYS_NAME Qsys creation failed"
qsys-generate --synthesis=VERILOG $QSYS_NAME.qsys || fail "Qsys system generation failed"

##For Display Port GHRD only
qsys-script --script=ip/diffin/construct_diffin.tcl --quartus-project=$QUARTUS_NAME || fail "diffin Qsys creation failed"
qsys-generate --synthesis=VERILOG diffin.qsys --output-directory=./diffin/ || fail "diffin Qsys generation failed"

#quartus_map $QUARTUS_NAME.qpf || fail "Quartus map failed"
#quartus_sh -r -t "$QSYS_NAME/synthesis/submodules/hps_sdram_p0_pin_assignments.tcl" $QUARTUS_NAME.qpf || "Quartus tcl execution failed"
quartus_sh --flow compile $QUARTUS_NAME.qpf || fail "Quartus full compilation failed"

#Partial Reconfiguration Specific
quartus_cdb $QUARTUS_NAME -c $QUARTUS_NAME --export_partition root_partition --snapshot final --file base_static.qdb  --preserve_sdc || fail "Base Root Partition export failed"
quartus_sh --script=create_pr_revision.tcl -projectname $QUARTUS_NAME -revision $QUARTUS_NAME -pr_revision alternate_persona -pr_partition pr_r0_per || fail "PR persona implement revision creation failed"
qsys-script --script=construct_subsys_pr_region.tcl --quartus-project=$QUARTUS_NAME --rev=alternate_persona --cmd="set sub_qsys_pr pr_r0_per; set pr_persona 1" || fail "Persona PR Region 0 Subqsys system creation failed"
qsys-generate --synthesis=VERILOG pr_r0_per.qsys || fail "Persona PR Region Subqsys system generation failed"
quartus_sh --flow compile $QUARTUS_NAME.qpf -c alternate_persona|| fail "Quartus full compilation failed"

## OLD PR FLOW
#quartus_syn $QUARTUS_NAME -c alternate_persona || fail "Quartus PR Persona implement revision compilation failed"
#quartus_fit $QUARTUS_NAME -c alternate_persona || fail "Quartus PR Persona implement revision fitter compilation failed"
#quartus_sta $QUARTUS_NAME -c alternate_persona || fail "Quartus PR Persona implement revision sta compilation failed"
#quartus_asm $QUARTUS_NAME -c alternate_persona || fail "Quartus PR Persona implement revision asm compilation failed"

# #quartus_sh --script=create_pr_revision.tcl -projectname $QUARTUS_NAME -revision $QUARTUS_NAME -pr_revision synth_v2 -revision_type synthesis -pr_partition pr_r0_per || fail "PR persona 0 synthesis revision creation failed"
# #quartus_sh --script=create_pr_revision.tcl -projectname $QUARTUS_NAME -revision $QUARTUS_NAME -pr_revision synth_v3 -revision_type synthesis -pr_partition pr_r1_per || fail "PR persona 1 synthesis revision creation failed"
# quartus_sh --script=create_pr_revision.tcl -projectname $QUARTUS_NAME -revision $QUARTUS_NAME -pr_revision alternate_persona || fail "PR persona implement revision creation failed"
# qsys-script --script=construct_subsys_pr_region.tcl --quartus-project=$QUARTUS_NAME --rev=synth_v2 --cmd="set sub_qsys_pr pr_r0_per; set pr_persona 1" || fail "Persona PR Region 0 Subqsys system creation failed"
# qsys-generate --synthesis=VERILOG pr_r0_per.qsys || fail "Persona PR Region Subqsys system generation failed"
# #qsys-script --script=construct_subsys_pr_region.tcl --quartus-project=$QUARTUS_NAME --rev=synth_v3 --cmd="set sub_qsys_pr pr_r1_per; set pr_persona 1; set pr_region_id_switch 1" || fail "Persona PR Region 1 Subqsys system creation failed"
# #qsys-generate --synthesis=VERILOG pr_r1_per.qsys || fail "Persona PR Region Subqsys system generation failed"
# quartus_syn $QUARTUS_NAME -c synth_v2 || fail "PR Persona 0 Synthesis Revision compilation failed"
# quartus_cdb $QUARTUS_NAME -c synth_v2 --export_block root_partition --file synth_v2.qdb --snapshot synthesized || fail "PR Region 0 Synthesis Partition export failed"
# #quartus_syn $QUARTUS_NAME -c synth_v3 || fail "PR Persona 1 Synthesis Revision compilation failed"
# #quartus_cdb $QUARTUS_NAME -c synth_v3 --export_block root_partition --file synth_v3.qdb --snapshot synthesized || fail "PR Region 1 Synthesis Partition export failed"
# quartus_cdb $QUARTUS_NAME -c alternate_persona --import_block root_partition --file base_static_region.qdb || fail "Base Root Partition import failed"
# quartus_cdb $QUARTUS_NAME -c alternate_persona --import_block pr_partition_0 --file synth_v2.qdb || fail "PR Region 0 Partition import failed"
# #quartus_cdb $QUARTUS_NAME -c alternate_persona --import_block pr_partition_1 --file synth_v3.qdb || fail "PR Region 1 Partition import failed"
# quartus_fit $QUARTUS_NAME -c alternate_persona || fail "Quartus PR Persona implement revision fitter compilation failed"
# quartus_sta $QUARTUS_NAME -c alternate_persona || fail "Quartus PR Persona implement revision sta compilation failed"
# quartus_asm $QUARTUS_NAME -c alternate_persona || fail "Quartus PR Persona implement revision asm compilation failed"
# # quartus_sh -t flow.tcl || fail "Quartus PR flow TCL failed"

quartus_cpf -c ./output_files/$QUARTUS_NAME.pr_partition_0.pmsf ./output_files/$QUARTUS_NAME.pr_partition_0.rbf || fail "Base Persona PR Region 0 pmsf to rbf conversion failed"
quartus_cpf -c ./output_files/alternate_persona.pr_partition_0.pmsf ./output_files/alternate_persona.pr_partition_0.rbf || fail "PR Persona Region 0 pmsf to rbf conversion failed"
#quartus_cpf -c ./output_files/$QUARTUS_NAME.pr_partition_1.pmsf ./output_files/$QUARTUS_NAME.pr_partition_1.rbf || fail "Base Persona PR Region 1 pmsf to rbf conversion failed"
#quartus_cpf -c ./output_files/alternate_persona.pr_partition_1.pmsf ./output_files/alternate_persona.pr_partition_1.rbf || fail "PR Persona Region 1 pmsf to rbf conversion failed"
